#
# Online Forum Product
#
# (C) Copyright 1998,1999,2000,2001 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
from ImageFile import ImageFile
from Globals import DTMLFile

from Persistence import Persistent
from Globals import HTMLFile, HTML
from OFS.Folder import Folder

from Products.Squib.ForumSpringboard.ForumSpringboard import manage_addForumSpringboard, manage_addForumSpringboardForm, ForumSpringboard, addCategoryForm, addCategory

def manage_addForumDirectory(self, id, title='', REQUEST={}):
	""" Add a Forum Directory """

	o = ForumDirectory(id, title)
	self._setObject(id, o)
	o.postInitialisation()

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

manage_addForumDirectoryForm=DTMLFile('manage_addForumDirectoryForm', globals())

class ForumDirectory(Folder):
	""" foo """

	meta_type='Squib Forum Directory'

	manage_addForumSpringboard    =manage_addForumSpringboard
	manage_addForumSpringboardForm=manage_addForumSpringboardForm

	#
	# User Version
	#
	addCategoryForm=addCategoryForm
	addCategory=addCategory

	meta_types=all_meta_types=(({'name':ForumSpringboard.meta_type,
								'action':'manage_addForumSpringboardForm'},))



	index_html=DTMLFile('index_html', globals())
	forumDirectorySummaryLines=DTMLFile('forumDirectorySummaryLines', globals())
	configureForumDirectoryForm=DTMLFile('configureForumDirectoryForm', globals())

	userPreferencesForm=DTMLFile('userPreferencesForm', globals())
	chooseAvatarForm=DTMLFile('chooseAvatarForm', globals())
	allowOverride=1
	allowBBS=1
	bbMsg=['highlighting', 'urls', 'email', 'images']
	allowBBSSig=1
	bbSig=['highlighting', 'email']
	
	def __init__(self, id, title):
		self.id=id
		self.title=title

	def postInitialisation(self):
		self.manage_addFolder(id='smilies', title='Smilie Folder')
		self.manage_addFolder(id='avatars', title='Avatar Folder')

	def getSpringboards(self):
		""" Get a list of child Forums """
		return self.objectIds('Squib Forum Springboard')

	def getBasePath(self):
		return self.getPhysicalPath()

	def getBreadCrumbs(self, destPath):
		x = self.getBasePath()
		url = self.absolute_url()
		xl = len(x)
		tail=''
		bc=[]
		bc.append({'label':destPath[xl-1], 'url':url})
		for a in destPath[xl:-1]:
			tail=tail+'/'+a
			bc.append({'label':a, 'url':url+tail})
		return bc

	def getBreadCrumbTrail(self):
		return []

	def chooseAvatar(self, avatar, REQUEST):
		""" Setup an avatar """
		REQUEST.AUTHENTICATED_USER.setProperty('avatar', avatar)
		return self.index_html(self, REQUEST)

	def updateUserPreferences(self, REQUEST):
		""" Update User Preferences """
		u=REQUEST.AUTHENTICATED_USER
		u.setProperty('realname', REQUEST['user_realname'])
		u.setProperty('email', REQUEST['user_email'])
		u.setProperty('signature', REQUEST['user_signature'])

		return self.userPreferencesForm(self, REQUEST)
			
	def configureForumDirectory(self, allowOverride=0, allowBBS=0,
								bbMsg=[], allowBBSSig=0,
								bbSig=[], REQUEST={}):

		""" Configure the Forum from here down """

		self.allowOverride=allowOverride
		self.allowBBS=allowBBS
		self.bbMsg=bbMsg
		self.allowBBSSig=allowBBSSig
		self.bbSig=bbSig

		if REQUEST:
			return self.index_html(self, REQUEST)
		return ''

	def isBBAllowed(self):
		""" Is BB Script allowed in messages? """
		return self.allowBBS

	def isBBSigAllowed(self):
		""" Is BB Script allowed in sigs? """
		return self.allowBBSSig

	def bbMsgFeatureAllowed(self, feature):
		""" Is a feature available? """
		return (feature in self.bbMsg)

	def bbSigFeatureAllowed(self, feature):
		""" Is a feature allowed in sigs? """
		return (feature in self.bbSig)
	
