#
# Online Forum Product
#
# (C) Copyright 1998,1999,2000,2001 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>
import os
import App, Globals, OFS
import string
from ImageFile import ImageFile
from Globals import DTMLFile

from Persistence import Persistent
from Products.ZSQLMethods.SQL import SQL
from Globals import HTMLFile, HTML, ImageFile
from OFS.Folder import Folder
from Acquisition import aq_base

from Products.Squib.Topics.Topics import Topic, manage_addTopic, manage_addTopicForm, addTopicForm, addTopic
from Products.Squib.Topics.TopicsQueries import _sqlGetForumTopics

from ForumQueries import _sqlAddNewForum


def manage_addForum(self, id, title, moderator, connection, REQUEST={}):
	""" Add a New Forum """
	o = Forum(id, title, moderator, connection)
	self._setObject(id, o)
	o = getattr(self, id)
	o.postInitialisation()

	#
	# Give the moderator, Moderator Priveleges
	#
	o.manage_addLocalRoles(userid=moderator, roles=['Moderator'])

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

def addForum(self, id, title, moderator, connection, REQUEST={}):
	""" Add a New Forum """
	o = Forum(id, title, moderator, connection)
	self._setObject(id, o)
	o = getattr(self, id)
	o.postInitialisation()

	#
	# Give the moderator, Moderator Priveleges
	#
	o.manage_addLocalRoles(userid=moderator, roles=['Moderator'])

	if REQUEST:
		return self.index_html(self, REQUEST)
	return ''

manage_addForumForm=DTMLFile('manage_addForumForm',globals())
addForumForm=DTMLFile('addForumForm',globals())

class Forum(Folder):
	""" A forum is a topics manager, with some simple operations """

	meta_type='Squib Forum'

	manage_addTopic=manage_addTopic
	manage_addTopicForm=manage_addTopicForm

	addTopic=addTopic
	addTopicForm=addTopicForm

	meta_types=all_meta_types=(({'name':Topic.meta_type,
								'action':'manage_addTopicForm'},))

	__ac_permissions__=(
		('View management screens',
		 ['manage','manage_menu','manage_main','manage_copyright',
		  'manage_tabs', 'manage_properties', 'manage_UndoForm']),
		('Undo changes',       ['manage_undo_transactions']),
		('Change permissions', ['manage_access']),
		('Add Forum Topic', ['manage_addTopic', 'manage_addTopicForm',],
		 ('Moderator',)),
		)

	index_html=DTMLFile('index_html', globals())

	forumSummary=DTMLFile('forumSummary', globals())
	topicSummaryLineItems=DTMLFile('topicSummaryLineItems', globals())
	configureForumForm=DTMLFile('configureForumForm', globals())

	vbar_gif=ImageFile('../images/vbar_gif', globals())
	lbar_gif=ImageFile('../images/lbar_gif', globals())
	tbar_gif=ImageFile('../images/tbar_gif', globals())
	boxed_minus_gif=ImageFile('../images/boxed_minus_gif', globals())
	boxed_plus_gif=ImageFile('../images/boxed_plus_gif', globals())

	lastPostDate=0.0

	def __init__(self, id, title, moderator, connection,
				 category=''):
		self.id=id
		self.title=title
		self.moderator=moderator
		self.connection=connection
		self.category=category
		self.addSQLQueries()

	def setLastPostDate(self, when):
		self.lastPostDate=when
				
	def addSQLQueries(self):
		self.sqlAddNewForum=SQL(
			'sqlAddNewForum',
			'Add a New Forum',
			self.connection,
			'forumId title moderator',
			_sqlAddNewForum)

		self.sqlGetForumTopics=SQL(
			'sqlGetTopicListing',
			'Get Topics for this Forum',
			self.connection,
			'forumId',
			_sqlGetForumTopics)

	def __setstate__(self, state):
		Persistent.__setstate__(self, state)
		if hasattr(self, 'smilies'):
			if type(self.smilies) == type({}):
				del self.smilies
		self.addSQLQueries()
	
	def getTopics(self):
		""" Get a list of topics under this Forum """
		return self.objectIds('Squib Topic')

	def manage_editForum(self):
		pass

	def postInitialisation(self):
		# Avoid Acquisition hassles
		self.sqlAddNewForum(forumId=self.id,
							title=self.title,
							moderator=self.moderator)

		if self.aq_parent.meta_type != 'Squib Forum Springboard':
			self.manage_addFolder(id='smilies', title='Smilie Folder')
			self.manage_addFolder(id='avatars', title='Avatar Folder')
			self.allowOverride=1
			self.allowBBS=1
			self.bbMsg=['highlighting', 'urls', 'email', 'images']
			self.allowBBSSig=1
			self.bbSig=['highlighting', 'email']

	def getBreadCrumbs(self, destPath):
		x = self.getBasePath()
		url = self.absolute_url()
		xl = len(x)
		tail=''
		bc=[]
		bc.append({'label':destPath[xl-1], 'url':url})
		for a in destPath[xl:-1]:
			tail=tail+'/'+a
			bc.append({'label':a, 'url':url+tail})
		return bc

	def getBasePath(self):
		#
		# Acquire first, then use ours.
		#
		if hasattr(self.aq_parent, 'getBasePath'):
			return self.aq_parent.getBasePath()
		return self.getPhysicalPath()
	
	def getBreadCrumbTrail(self):
		#
		# Preferred acquired one first.
		#
		if hasattr(self.aq_parent, 'getBreadCrumbs'):
			return self.aq_parent.getBreadCrumbs(self.getPhysicalPath())
		
		return self.getBreadCrumbs(self.getPhysicalPath())

	def configureForum(self, allowBBS=0,
					   bbMsg=[], allowBBSSig=0,
					   bbSig=[], REQUEST={}):

		""" Configure the Forum from here down """

		if ((not hasattr(self.aq_parent,'allowOverride')) or (
			self.aq_parent.allowOverride)):
			
			self.allowBBS=allowBBS
			self.bbMsg=bbMsg
			self.allowBBSSig=allowBBSSig
			self.bbSig=bbSig

		if REQUEST:
			return self.index_html(self, REQUEST)
		return ''

	def isBBAllowed(self):
		""" Is BB Script allowed in messages? """
		return self.allowBBS

	def isBBSigAllowed(self):
		""" Is BB Script allowed in sigs? """
		return self.allowBBSSig

	def bbMsgFeatureAllowed(self, feature):
		""" Is a feature available? """
		return (feature in self.bbMsg)

	def bbSigFeatureAllowed(self, feature):
		""" Is a feature allowed in sigs? """
		return (feature in self.bbSig)
