#
# Online Forum Product
#
# (C) Copyright 1998,1999,2000,2001 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
from ImageFile import ImageFile
from Globals import Acquisition
from Persistence import Persistent
from Products.ZSQLMethods.SQL import SQL
from Globals import DTMLFile, HTMLFile, HTML
from OFS.Folder import Folder

from time import time

manage_addPostingForm=DTMLFile('manage_addPostingForm', globals())
newThreadForm=DTMLFile('newThreadForm', globals())

def manage_addPosting(self, title, body, smiley='', REQUEST={}):
	""" Add a new posting """

	poster=REQUEST['AUTHENTICATED_USER'].getUserName()
	pu=REQUEST['AUTHENTICATED_USER']
	signature=''
	if pu.hasProperty('signature'):
		signature=pu['signature']

	self.sqlAddNewPosting(forumId=self.forumId,
						  topicId=self.id,
						  title=title,
						  body=body,
						  poster=poster,
						  signature=signature,
						  messageIcon=smiley)

	self.postingCount=self.postingCount+1
	self.setLastPostDate(time())

	return self.manage_main(self, REQUEST)

def startThread(self, title, body, smiley='', REQUEST={}):
	""" Start a New Thread """
	self.manage_addPosting(title, body, smiley, REQUEST)
	REQUEST.RESPONSE.redirect('index_html')

#
# Postings aren't stored in the ZODB... we load them when we need them.
# They get removed from memory when they're flushed out by Zope
#
class Posting(Folder,Acquisition.Implicit):

	meta_type="Squib Posting"

	title=''
	body=''
	poster=''
	postedon=''
	numReplies=0

	index_html=DTMLFile('index_html', globals())
	tree_index_html=DTMLFile('treeIndex_html', globals())
	flat_index_html=DTMLFile('flatIndex_html', globals())	

	__ac_permissions__=(
		('View', ['manage_addReplyForm', 'manage_addReply']),
		)
	manage_addReplyForm=DTMLFile('manage_addReplyForm', globals())
	replyForm=DTMLFile('replyForm', globals())
	quotedReplyForm=DTMLFile('quotedReplyForm', globals())
	postingSummary=DTMLFile('postingSummary', globals())
	displayMessage=DTMLFile('displayMessage', globals())
	displaySelectedMessage=DTMLFile('displaySelectedMessage', globals())
	editPostingForm=DTMLFile('editForm', globals())

	__ac_local_roles__=Acquisition.Acquired # We don't exist get roles from above
	
	def __init__(self, id):
		self.id=id
		self.postingId=int(id)

	def loadPosting(self):
		result=self.sqlLoadPosting(postId=self.postingId)
		for x in result:
			self.title=sqlattr(x, 'title')
			self.body=sqlattr(x, 'body')
			self.poster=sqlattr(x, 'poster')
			self.postedon=sqlattr(x,'postedon')
			self.inReplyTo=sqlattr(x,'inreplyto')
			self.messageIcon=sqlattr(x, 'icon')
			self.lastPostDate=sqlattr(x, 'lastpostdate')
			self.replyCount=sqlattr(x, 'replycount')
			self.messageViews=sqlattr(x, 'viewcount')
			self.editor=sqlattr(x, 'editor')
			self.editDate=sqlattr(x, 'editdate')
			self.signature=sqlattr(x, 'signature')
			if not self.messageViews:
				self.messageViews=0
			if not self.replyCount:
				self.replyCount=0
			if not self.signature:
				self.signature=''
				
	editPosting__roles__=('Authenticated',)
	def editPosting(self, title, body, REQUEST, smiley=''):
		""" Edit a Posting """
		u=REQUEST['AUTHENTICATED_USER']
		editor=REQUEST['AUTHENTICATED_USER'].getUserName()
		if ( (editor != self.poster) and
			 not (editor.has_role(['Manager', 'Moderator']))):
			# This ain't a wiki...
			return self.index_html(self, REQUEST)
			
		self.sqlUpdatePosting(forumId=self.forumId,
							  topicId=self.topicId,
							  title=title,
							  body=body,
							  editor=editor,
							  editDate=time(),
							  inReplyTo=self.id,
							  messageIcon=smiley,
							  postId=self.postingId)
		self.loadPosting()
		if self.inReplyTo:
			return self.aq_parent.index_html(self, REQUEST)
		else:
			return self.index_html(self, REQUEST)

	removeThread__roles__=('Manager', 'Moderator')
	def removeThread(self, REQUEST):
		""" Remove an Entire Thread """
		for i in self.objectValues():
			i.removeThread(REQUEST)
		self.sqlRemovePosting(postId=self.postingId)
		return self.aq_parent.index_html(self, REQUEST)
			
	removeSignature__roles__=('Manager', 'Moderator')
	def removeSignature(self, REQUEST):
		""" Remove a Signature from a Posting """
		self.sqlRemoveSignature(postId=self.postingId)
		if self.inReplyTo:
			return self.aq_parent.index_html(self, REQUEST)
		else:
			return self.index_html(self, REQUEST)

	manage_addReply__roles__=('Authenticated',)
	def manage_addReply(self, title, body, REQUEST, smiley=''):
		""" Add a Reply """

		poster=REQUEST['AUTHENTICATED_USER'].getUserName()
		pu=REQUEST['AUTHENTICATED_USER']
		signature=''
		if pu.hasProperty('signature'):
			signature=pu['signature']
		
		self.sqlAddNewReply(forumId=self.forumId,
							topicId=self.topicId,
							title=title,
							body=body,
							signature=signature,
							poster=poster,
							inReplyTo=self.id,
							messageIcon=smiley)

		self.postingCount=self.postingCount+1
		self.setLastPostDate(time())
		self.updateReplyCount()
		return self.manage_main(self, REQUEST)

	def drawPostingTreeView(self, level=0, noexpand=0, final=0, BASE_URL=''):
		""" Draw the Tree View """

		if not level and not BASE_URL:
			BASE_URL=self.absolute_url()

		a = self.drawTreeView(level, final, noexpand, BASE_URL)
		if self.isExpanded(int(self.id)):
			subCount=len(self.objectIds())
			i=1
			zzz=self.objectValues()
			zzz.sort(lambda x,y: cmp(x.id, y.id))
			for p in zzz:
				if self.isExpanded(int(p.id)):
					if (i) == (len(self.objectIds())):
						noexpand = (noexpand | (1<<level))
				a = a + p.drawPostingTreeView(level+1, noexpand, (i==subCount), BASE_URL)
				i=i+1
		return a

	def getPostingPath(self):
		a=[]
		if self.inReplyTo:
			p=self.getPosting(self.inReplyTo)
			a=p.getPostingPath()
		a.append(self.id)
		return a

	style="font-size: 0.6em; text-decoration: none; font-family: Helvetica,Arial; color:Black;"
	selectedStyle="font-size: 0.6em; text-decoration: none; font-family: Helvetica,Arial; color:red;"	
	def drawTreeView(self, level=0, final=0, noexpand=0, BASE_URL=''):
		colspan=self.expandCount()+6
		i = 0
		a='<tr bgcolor="White">\n'
		for i in range(level-1):
			a=a+'<td align="left" valign="top" width="16" height="16">'
			if noexpand & (1<<i):
				a=a+'&nbsp;'
			else:
				a=a+str(self.vbar_gif)
			colspan=colspan-1
			a=a+'</td>\n'
		a=a+'<td align="left" valign="top" width="16" height="16" colspan="%d">\n'%(colspan)
		a=a+'<table width="%d" border="0" cellspacing="0" cellpadding="0">\n'%(16*colspan+50)
		a=a+'<tr bgcolor="White">\n'

		if level!=0:
			a=a+'<td align="left" valign="top" width="16" height="16">\n'
			if final:
				a=a+str(self.lbar_gif)
			else:
				a=a+str(self.tbar_gif)
			a=a+'</td>\n'
		
		# # # # #
		au = BASE_URL+"?viewmode=tree"
		ss = self.BuildForumStateTree(int(self.id), self.isExpanded(int(self.id)), 1)
		if self.isExpanded(int(self.id)):
			image=self.boxed_minus_gif.__name__
		else:
			image=self.boxed_plus_gif.__name__

		if self.messageIcon:
			iconImage='<img src="%s" border="0" height="15" width="15">'%(self.messageIcon)
		else:
			iconImage='<img src="none_gif" border="0" height="15" width="15">'
		

		if self.isSelected(int(self.id)):
			theStyle=self.selectedStyle
		else:
			theStyle=self.style

		if len(self.objectIds()):
			fs=self.BuildForumStateTree(int(self.id), not self.isExpanded(int(self.id)), 1)

			a=a+'<td><a href="%s%s"><img src="%s" border="0" height="12" width="12"></a></td>'%(au, fs, image)
			a=a+'<td>%s</td>'%(iconImage)
			a=a+'<td colspan="%d" NOWRAP width="100%%"> '%(colspan)
			a=a+'<a  style="%s" href="%s%s">%s</a>'%(theStyle,
													 au, ss,
													 self.title)
													 
			a=a+'</td>\n'
		else:
			a=a+'<td>%s</td>'%(iconImage)			
			a=a+'<td colspan="%d" NOWRAP width="100%%"> '%(colspan)
			a=a+'<a style="%s" href="%s%s">%s</a>'%(
													theStyle, au, ss,
													self.title)
			a=a+'</td>\n'					

		# # # # #
		a=a+'</tr>\n'
		a=a+'</table>\n'
		if self.replyCount:
			a=a+ '</td><td align="right" style="%s">(%d)'%(theStyle, self.replyCount)
			
		a=a+'</td>\n'
		a=a+'</tr>'
		return a

	def updateParentLastPostDate(self, when):
		o=self.getPosting(self.inReplyTo)
		o.setLastPostDate(when)

	def updateParentReplyCount(self):
		o=self.getPosting(self.inReplyTo)
		o.updateReplyCount()

	def decrementParentReplyCount(self):
		o=self.getPosting(self.inReplyTo)
		o.decrementReplyCount()

	def updateReplyCount(self):
		self.replyCount=self.replyCount+1
		self.sqlUpdateReplyCount(id=self.postingId)
		if self.inReplyTo:
			self.updateParentReplyCount()

	def decrementReplyCount(self):
		self.replyCount=self.replyCount-1
		self.sqlDecrementReplyCount(id=self.postingId)
		if self.inReplyTo:
			self.decrementParentReplyCount()
	
	def setLastPostDate(self, when):
		self.lastPostDate=when
		self.sqlUpdateLastPostDate(lastPostDate=when, id=self.postingId)
		if self.inReplyTo:
			self.updateParentLastPostDate(when)
		else:
			self.aq_parent.setLastPostDate(when)

	def updateViewStats(self):
		self.messageViews=self.messageViews+1
		self.sqlUpdateViewCount(id=self.postingId)

	def doThreadReplies(self):
		""" Show all replies in a flat manner """
		a=''
		for x in self.objectValues():
			a=a+x.displayMessage(self, self.REQUEST)
			if self.isExpanded(x.id):
				a=a+x.doThreadReplies()
		return a
		
	def doFlatReplies(self):
		""" Show all replies in a flat manner """
		a=''
		for x in self.objectValues():
			a=a+x.displayMessage(self, self.REQUEST)
			a=a+x.doFlatReplies()
		return a

	def replyToPost(self, title, body, REQUEST, smiley=''):
		""" Reply to a message """
		self.manage_addReply(title, body, REQUEST, smiley)
		REQUEST.RESPONSE.redirect('index_html')

	def _objects(self):
		o=[]
 		results=self.sqlGetPostingReplies(id=self.id)
		for i in results:
			p=Posting(str(sqlattr(i, 'postingkey')))
			p=p.__of__(self)
			p.loadPosting()
			o.append(p)
		return o

	def objectMap(self):
		# Return a tuple of mappings containing subobject meta-data
		return tuple(map(lambda dict: dict.copy(), self._objects()))

	def objectMap_d(self,t=None):
		if hasattr(self, '_reserved_names'): n=self._reserved_names
		else: n=()
		if not n: return self._objects()
		r=[]
		a=r.append
		for d in self._objects():
			if d['id'] not in n: a(d.copy())
		return r

	def getBreadCrumbTrail(self):
		return self.getBreadCrumbs(self.getPhysicalPath())

	def superValues(self,t):
		# Return all of the objects of a given type located in
		# this object and containing objects.
		if type(t)==type('s'): t=(t,)
		obj=self
		seen={}
		vals=[]
		have=seen.has_key
		x=0
		while x < 100:
			if not hasattr(obj,'_getOb'): break
			get=obj._getOb
			if hasattr(obj,'_objects'):
				for i in obj._objects():
					try:
						id=i['id']
						if (not have(id)) and (i['meta_type'] in t):
							vals.append(get(id))
							seen[id]=1
					except: pass

			if hasattr(obj,'aq_parent'): obj=obj.aq_parent
			else:						 return vals
			x=x+1
		return vals

	def objectIds(self, spec=None):
		# Returns a list of subobject ids of the current object.
		# If 'spec' is specified, returns objects whose meta_type
		# matches 'spec'.
		if spec is not None:
			if type(spec)==type('s'):
				spec=[spec]
			set=[]
			for ob in self._objects():
				if ob['meta_type'] in spec:
					set.append(ob['id'])
			return set
		return map(lambda i: i['id'], self._objects())

	def _getOb(self, id, default=[]):
		return self.getPosting(id)
				
	def __getitem__(self, key):
		try:
			if hasattr(self, key):
				return getattr(self, key)
		except:
			v = self.getPosting(key)
			if v is not None:
				return v

		raise AttributeError, key

from string import upper, lower
import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name
        

