#
# Online Forum Product
#
# (C) Copyright 1998,1999,2000,2001 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
from ImageFile import ImageFile

from Persistence import Persistent
from Products.ZSQLMethods.SQL import SQL
from Globals import DTMLFile, HTMLFile, HTML, ImageFile, Acquisition
from OFS.Folder import Folder

from Products.Squib.Postings.Postings import Posting, manage_addPosting, manage_addPostingForm, newThreadForm, startThread
from Products.Squib.Postings.PostingQueries import _sqlAddNewPosting, _sqlAddNewReply, _sqlLoadPosting, _sqlGetTopicPostings, _sqlGetPosting, _sqlGetPostingReplies, _sqlUpdateLastPostDate, _sqlUpdateReplyCount, _sqlUpdateViewCount, _sqlUpdatePosting, _sqlRemoveSignature, _sqlRemovePosting, _sqlDecrementReplyCount

import re
from TopicsQueries import _sqlAddTopic


def manage_addTopic(self, id, title, REQUEST):
	""" Add a new Topic """
	owner = REQUEST['AUTHENTICATED_USER'].getUserName()
	connection=self.connection

	o = Topic(id, self.id, title, owner, connection)
	self._setObject(id, o)
	o = getattr(self, id)
	o.postInitialisation()

	return self.manage_main(self, REQUEST)

def addTopic(self, id, title, REQUEST):
	""" Add a new Topic """
	owner = REQUEST['AUTHENTICATED_USER'].getUserName()
	connection=self.connection

	o = Topic(id, self.id, title, owner, connection)
	self._setObject(id, o)
	o = getattr(self, id)
	o.postInitialisation()

	return self.index_html(self, REQUEST)
	
manage_addTopicForm=DTMLFile('manage_addTopicForm', globals())
addTopicForm=DTMLFile('addTopicForm', globals())

class Topic(Folder):
	""" A Topic is a discussable item """

	meta_type='Squib Topic'
	manage_addPosting=manage_addPosting
	manage_addPostingForm=manage_addPostingForm
	newThreadForm=newThreadForm
	startThread=startThread

	meta_types=all_meta_types=(({'name':Posting.meta_type,
								'action':'manage_addPostingForm'},))

	postingCount=0
	lastPostDate=0.0

	topicSummary=DTMLFile('topicSummary', globals())
	postingSummaryLineItems=DTMLFile('postingSummaryLineItems', globals())
	index_html=DTMLFile('index_html', globals())
	__ac_local_roles__=Acquisition.Acquired # We don't exist get roles from above	

	def __init__(self, id, forumId, title, owner, connection):
		self.id=id
		self.topicId=id
		self.forumId=forumId
		self.title=title
		self.owner=owner
		self.connection=connection
		self.addSQLQueries()

		self.selected=[]
		self.expanded=[]
		for i in range(32):
			self.selected.append(0)
			self.expanded.append(0)

		self.selectedCount=0
		self.expandedCount=0

	def postInitialisation(self):
		self.sqlAddTopic(
			topicId=self.id,
			forumId=self.forumId,
			title=self.title,
			owner=self.owner)
	
	def addSQLQueries(self):
		self.sqlAddTopic=SQL(
			'sqlAddTopic',
			'Add a New Topic',
			self.connection,
			'topicId forumId title owner',
			_sqlAddTopic)

		self.sqlAddNewPosting=SQL(
			'sqlAddPosting',
			'Add a New Posting',
			self.connection,
			'topicId forumId title poster body messageIcon signature',
			_sqlAddNewPosting)

		self.sqlAddNewReply=SQL(
			'sqlAddReply',
			'Add a New Reply',
			self.connection,
			'topicId forumId title poster body inReplyTo messageIcon signature',
			_sqlAddNewReply)

		self.sqlLoadPosting=SQL(
			'sqlLoadPosting',
			'Get a Posting',
			self.connection,
			'postId',
			_sqlLoadPosting)

		self.sqlGetTopicPostings=SQL(
			'sqlLoadTopicPostings',
			'Get all Postings for a Topic',
			self.connection,
			'topicId',
			_sqlGetTopicPostings)

		self.sqlGetPosting=SQL(
			'sqlGetPosting',
			'Verify an ID',
			self.connection,
			'id',
			_sqlGetPosting)

		self.sqlGetPostingReplies=SQL(
			'sqlGetPostingReplies',
			'Verify a subID',
			self.connection,
			'id',
			_sqlGetPostingReplies)

		self.sqlUpdateLastPostDate=SQL(
			'sqlUpdateLastPostDate',
			'Update the last posting for a thread',
			self.connection,
			'lastPostDate id',
			_sqlUpdateLastPostDate)

		self.sqlUpdateReplyCount=SQL(
			'sqlUpdateReplyCount',
			'Update Count for Replies',
			self.connection,
			'id',
			_sqlUpdateReplyCount)

		self.sqlDecrementReplyCount=SQL(
			'sqlDecrementReplyCount',
			'Decerement Count for Replies',
			self.connection,
			'id',
			_sqlDecrementReplyCount)

		self.sqlUpdateViewCount=SQL(
			'sqlUpdateViewCount',
			'Update View Count',
			self.connection,
			'id',
			_sqlUpdateViewCount)

		self.sqlUpdatePosting=SQL(
			'sqlUpdatePosting',
			'Edit a Posting',
			self.connection,
			'topicId forumId title editor body messageIcon editDate postId',
			_sqlUpdatePosting)

		self.sqlRemoveSignature=SQL(
			'sqlRemoveSignature',
			'Remove a Signature from a Posting',
			self.connection,
			'postId',
			_sqlRemoveSignature)

		self.sqlRemovePosting=SQL(
			'sqlRemovePosting',
			'Remove a Posting',
			self.connection,
			'postId',
			_sqlRemovePosting)

	def __setstate__(self, state):
		Persistent.__setstate__(self, state)
		self.addSQLQueries()
		self.topicId=self.id
		self.selected=[]
		self.expanded=[]
		for i in range(32):
			self.selected.append(0)
			self.expanded.append(0)

		self.selectedCount=0
		self.expandedCount=0
		

	def _objects(self):
		o=[]
 		results=self.sqlGetTopicPostings(topicId=self.id)
		for i in results:
			p=Posting(str(sqlattr(i, 'postingkey')))
			p=p.__of__(self)
			p.loadPosting()
			o.append(p)
		return o

	def setLastPostDate(self, when):
		self.lastPostDate=when
		self.aq_parent.setLastPostDate(when)
		
	def selectAndExpand(self, REQUEST):
		for i in range(32):
			self.selected[i]=0
			self.expanded[i]=0
			self.selectedCount=0
			self.expandedCount=0
			
		if not REQUEST.has_key('forumTreeState'):
			return

		select,expand = string.split(REQUEST['forumTreeState'], '|')

		j = 0
		for i in string.split(select, ','):
			self.selected[j]=int(i)
			j = j + 1

		self.selectedCount=j
		j = 0
		if expand:
			for i in string.split(expand, ','):
				self.expanded[j]=int(i)
				j = j + 1
		self.expandedCount=j
		
	def isExpanded(self, id):
		return id in self.expanded

	def isSelected(self, id):
		return id in self.selected

	def expandCount(self):
		""" How many expanded Items """
		return self.expandedCount

	def selectCount(self):
		""" How many selected Items """
		return self.selectedCount

	def BuildForumStateTree(self, itemId, doexpand, doselected):
		hrefbuf="&forumTreeState="
		i = 0

		if doselected:
			tempbuf="%d"%(itemId)
		elif self.selectCount():
			tempbuf="%d"%(selected[0])

		hrefbuf=hrefbuf+tempbuf+"|"

		addcomma = 0
		if doexpand:
			addcomma=1
			hrefbuf=hrefbuf+("%d"%(itemId))

		while i < self.expandCount():
			if self.expanded[i]==itemId:
				i = i + 1
				continue
			if addcomma:
				hrefbuf=hrefbuf+","
			hrefbuf=hrefbuf+("%d"%(self.expanded[i]))
			addcomma=1
			i = i + 1
		return hrefbuf

	def objectMap(self):
		# Return a tuple of mappings containing subobject meta-data
		return tuple(map(lambda dict: dict.copy(), self._objects()))

	def objectMap_d(self,t=None):
		if hasattr(self, '_reserved_names'): n=self._reserved_names
		else: n=()
		if not n: return self._objects()
		r=[]
		a=r.append
		for d in self._objects():
			if d['id'] not in n: a(d.copy())
		return r

	def superValues(self,t):
		# Return all of the objects of a given type located in
		# this object and containing objects.
		if type(t)==type('s'): t=(t,)
		obj=self
		seen={}
		vals=[]
		have=seen.has_key
		x=0
		while x < 100:
			if not hasattr(obj,'_getOb'): break
			get=obj._getOb
			if hasattr(obj,'_objects'):
				for i in obj._objects():
					try:
						id=i['id']
						if (not have(id)) and (i['meta_type'] in t):
							vals.append(get(id))
							seen[id]=1
					except: pass

			if hasattr(obj,'aq_parent'): obj=obj.aq_parent
			else:						 return vals
			x=x+1
		return vals

	def objectIds(self, spec=None):
		# Returns a list of subobject ids of the current object.
		# If 'spec' is specified, returns objects whose meta_type
		# matches 'spec'.
		if spec is not None:
			if type(spec)==type('s'):
				spec=[spec]
			set=[]
			for ob in self._objects():
				if ob['meta_type'] in spec:
					set.append(ob['id'])
			return set
		return map(lambda i: i['id'], self._objects())

	def _getOb(self, id, default=[]):
		return self.getPosting(id)

	__old_getitem__=Folder.__getitem__
	def __getitem__(self, key):
		try:
			v = __old_getitem__(key)
		except:
			v = self.getPosting(key)
			if v is not None:
				return v

		raise AttributeError, key

	def getPostings(self):
		""" Get a list of postings """
		return self.objectIds('Squib Posting')

	def getPosting(self, name):
		results=self.sqlGetPosting(id=name)
		if not results:
			raise AttributeError, key

		if not len(results):
			raise AttributeError, key

		p=Posting(name)
		p=p.__of__(self) # Give it a wrapper
		p.loadPosting()
		return(p)

	def selectedPost(self):
		""" Return the selected Post Id """
		return str(self.selected[0])
	
	def getSelectedPostingPath(self):
		""" Display the currently selected posting """
		p=Posting(str(self.selected[0]))
		p=p.__of__(self) # Give it a wrapper
		p.loadPosting()
		return p.getPostingPath()

	def getBreadCrumbTrail(self):
		return self.getBreadCrumbs(self.getPhysicalPath())

	def filterSignature(self, text):
		text = re.sub("&","&amp;", text)
		text = re.sub("<","&lt;", text)
		text = re.sub(">","&gt;", text)
		text = re.sub("\n","<br>", text)
		# Start inline processing

		if self.isBBSigAllowed():
			if self.bbSigFeatureAllowed('highlighting'):
				# Bold
				text = re.sub("\[b\]","<b>", text)
				text = re.sub("\[/b\]","</b>", text)

				# Italic
				text = re.sub("\[i\]","<i>", text)
				text = re.sub("\[/i\]","</i>", text)

				#Underline
				text = re.sub("\[u\]","<u>", text)
				text = re.sub("\[/u\]","</u>", text)

			# Additional features
			if self.bbSigFeatureAllowed('urls'):
				## Links
				text = re.sub("\\[url\\]www.([^\\[]*)\\[/url\\]", "<a href=\"http://www.\\1\" target=_blank>\\1</a>", text)
				text = re.sub("\\[url\\]([^\\[]*)\\[/url\\]","<a href=\"\\1\" target=_blank>\\1</a>", text)
				text = re.sub("\\[url=([^\\[]*)\\]([^\\[]*)\\[/url\\]","<a href=\"\\1\" target=_blank>\\2</a>", text)

			if self.bbSigFeatureAllowed('email'):
				## Email
				text = re.sub("\\[email\\]([^\\[]*)\\[/email\\]", "<a href=\"mailto:\\1\">\\1</a>", text)
			if self.bbSigFeatureAllowed('images'):
				## Images
				text = re.sub("\\[img\\]([^\\[]*)\\[/img\\]","<img src=\"\\1\" border=0>", text)

		return text

	def filterMessageBody(self, text):
		text = re.sub("&","&amp;", text)
		text = re.sub("<","&lt;", text)
		text = re.sub(">","&gt;", text)
		text = re.sub("\n","<br>", text)
		# Start inline processing

		if self.isBBAllowed():
			if self.bbMsgFeatureAllowed('highlighting'):
				# Bold
				text = re.sub("\[b\]","<b>", text)
				text = re.sub("\[/b\]","</b>", text)

				# Italic
				text = re.sub("\[i\]","<i>", text)
				text = re.sub("\[/i\]","</i>", text)

				#Underline
				text = re.sub("\[u\]","<u>", text)
				text = re.sub("\[/u\]","</u>", text)

			# Additional features
			if self.bbMsgFeatureAllowed('urls'):
				## Links
				text = re.sub("\\[url\\]www.([^\\[]*)\\[/url\\]", "<a href=\"http://www.\\1\" target=_blank>\\1</a>", text)
				text = re.sub("\\[url\\]([^\\[]*)\\[/url\\]","<a href=\"\\1\" target=_blank>\\1</a>", text)
				text = re.sub("\\[url=([^\\[]*)\\]([^\\[]*)\\[/url\\]","<a href=\"\\1\" target=_blank>\\2</a>", text)

			if self.bbMsgFeatureAllowed('email'):
				## Email
				text = re.sub("\\[email\\]([^\\[]*)\\[/email\\]", "<a href=\"mailto:\\1\">\\1</a>", text)
			if self.bbMsgFeatureAllowed('images'):
				## Images
				text = re.sub("\\[img\\]([^\\[]*)\\[/img\\]","<img src=\"\\1\" border=0>", text)
			## Quotes
			text = re.sub("\[quote\]","<BLOCKQUOTE><SMALLFONT>quote:</SMALLFONT><HR>", text)
			text = re.sub("\[/quote\]","<hr></blockquote>", text)

		return text
		
		

from string import upper, lower
import Missing
mt=type(Missing.Value)

def typeconv(val):
    if type(val)==mt:
        return ''
    return val

def sqlattr(ob, attr):
    name=attr
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=upper(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    attr=lower(attr)
    if hasattr(ob, attr):
        return typeconv(getattr(ob, attr))
    raise NameError, name
        

